<?php
/**
 * Plugin Name: OpoChat
 * Plugin URI: http://wordpress.org/plugins/opochat/
 * Description: OpoChat provides a comprehensive live chat widget solution with an AI chatbot, CRM, help center, and more. Seamlessly integrated with WordPress and WooCommerce, it automatically displays the names and emails of logged-in visitors. Create a free account or log in with an existing one to get started.
 * Author: OpoChat
 * Version: 1.0.1
 * Author URI: https://opo.chat
 * License: MIT
 * License URI: https://opensource.org/licenses/MIT
 * Text Domain: opochat
*/

//https://shop.opo.chat/wp-admin/admin.php?page=opochat%2Fopochat.php&_wpnonce=583a1abfd9&website_apiKey=opochat8ce9bc8d15bf0dc07d78f2cd88&opochat_verify
require_once( ABSPATH . 'wp-admin/includes/plugin.php' );

// Notice: if the woocommerce plugin is installed, deactivate the OpoChat WP \
//plugin to prevent any duplicate widgets and OpoChat menu.
add_action(
  'plugins_loaded',
  function () {
    if (defined("WP_OPOCHAT_WOOCOMMERCE")) {
      register_deactivation_hook( __FILE__ ,  function() {
        delete_option("website_apiKey");
        delete_option("opochat_onboarding");
        delete_option("website_verify");
      });

      deactivate_plugins(plugin_basename(__FILE__));

      return;
    } else {
      add_action("admin_menu", "opochat_create_menu");
      add_action("wp_enqueue_scripts", "opochat_enqueue_scripts");
    }
  }
);

function opochat_create_menu() {
  add_menu_page(
    __("OpoChat", "opochat"),
    __("OpoChat", "opochat"),
    "administrator", __FILE__,
    "opochat_plugin_settings_page" ,
    plugins_url("/assets/images/icon-favicon-16x16.png", __FILE__)
  );

  add_action("admin_init", "register_opochat_plugin_settings" );
  add_action("admin_init", "register_opochat_plugin_onboarding");
  add_action("admin_notices", "register_opochat_plugin_notice");
  add_action("admin_enqueue_scripts", "register_opochat_admin_enqueue");
  add_action("plugins_loaded", "register_opochat_plugin_textdomain" );
}

function register_opochat_plugin_textdomain() {
  load_plugin_textdomain("opochat", false, dirname(plugin_basename(__FILE__)) . "/languages/");
}

function register_opochat_plugin_onboarding() {
  $onboarding = get_option("opochat_onboarding");
  $website_apiKey = get_option("website_apiKey");

  if (empty($website_apiKey) && (empty($onboarding) || !$onboarding)) {
    update_option("opochat_onboarding", true);
    wp_redirect(admin_url("admin.php?page=".plugin_basename(__FILE__)));
  }
}

function register_opochat_plugin_notice() {
  $website_apiKey = get_option("website_apiKey");

  if (empty($website_apiKey) && (!isset($_GET["page"]) || $_GET["page"] != plugin_basename(__FILE__))) {
    $admin_url = admin_url("admin.php?page=".plugin_basename(__FILE__));
    ?>
      <div class="notice notice-warning is-dismissible notice-opochat">
        <p>
          <img src="<?php echo esc_url("/assets/images/icon-favicon-16x16.png", __FILE__); ?>" height="16" style="margin-bottom: -3px" />
          &nbsp;
          <?php 
            echo sprintf(
                // translators: %1$s is the opening <a> tag for the admin URL, %2$s is the closing </a> tag.
                esc_html__('The OpoChat plugin isn’t connected right now. To display OpoChat widget on your WordPress site, %1$sconnect the plugin now%2$s. The configuration takes less than 1 minute!', 'opochat'),
                '<a href="' . esc_url($admin_url) . '">', // Escape URL and wrap in <a> tag
                '</a>'
            );
            ?>
        </p>
      </div>
    <?php
  }
}

function register_opochat_plugin_settings() {
  register_setting("opochat-plugin-settings-group", "website_apiKey", array(
    'sanitize_callback' => 'sanitize_text_field'
  ));  
  add_option("opochat_onboarding", false);
}

function register_opochat_admin_enqueue() {
  wp_enqueue_style(
    'admin_opochat_style', 
    plugins_url('/assets/stylesheets/style.css', __FILE__), 
    array(), // 依赖
    '1.0.3'   // 版本号
  );
}

function opochat_plugin_settings_page() {
  add_action("admin_enqueue_scripts", "opochat_admin_enqueue");

  if (current_user_can("administrator")) {
    if(isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'])) {
      if (isset($_GET["website_apiKey"]) && !empty($_GET["website_apiKey"])) {

        $api_key = sanitize_text_field($_GET["website_apiKey"]);
        if (!empty($api_key)) {
            update_option("website_apiKey", $api_key);
            // 防止刷新重复写入，可以重定向（清除 URL 参数）
            wp_redirect(remove_query_arg(array("website_apiKey", "opochat_verify")));
            exit;
        }

        update_option("website_apiKey", sanitize_text_field($_GET["website_apiKey"]));

        // Clear WP Rocket Cache if needed
        if (function_exists("rocket_clean_domain")) {
          rocket_clean_domain();
        }

        // Clear WP Super Cache if needed
        if (function_exists("wp_cache_clean_cache")) {
          global $file_prefix;
          wp_cache_clean_cache($file_prefix, true);
        }
      }

      if (isset($_GET["opochat_verify"]) && !empty($_GET["opochat_verify"])) {
          update_option("website_verify", sanitize_key($_GET["opochat_verify"]));
      }
    }
  }

  $website_apiKey = get_option("website_apiKey");
  //$is_opochat_working = isset($website_apiKey) && !empty($website_apiKey);
  $is_opochat_working = false;
  if (isset($_GET["website_apiKey"]) && !empty($_GET["website_apiKey"])){
      update_option("website_apiKey", sanitize_text_field($_GET["website_apiKey"]));
      $is_opochat_working = true;
  }else if (isset($website_apiKey) && !empty($website_apiKey)) {
      $is_opochat_working = true;
  }
  $http_callback = "http" . (($_SERVER["SERVER_PORT"] == 443) ? "s://" : "://") . $_SERVER["HTTP_HOST"] . $_SERVER["REQUEST_URI"];
  $http_callback = urlencode(wp_nonce_url($http_callback));
  $admin_email = wp_get_current_user()->user_email;
  $admin_name = wp_get_current_user()->display_name;
  $website_name = get_bloginfo("name");
  $website_domain = get_site_url();
  $website_domain = str_replace("http://", "", $website_domain);
  $website_domain = str_replace("https://", "", $website_domain);
  $website_domain = preg_replace("(:[0-9]{1,6})", "", $website_domain);
  $add_to_opochat_link = esc_url_raw("https://app.opo.chat/#/index/integrations/wordpress?payload=$http_callback&user_email=$admin_email&user_name=$admin_name&website_name=$website_name&website_domain=$website_domain");

  if ($is_opochat_working) {
    include_once(plugin_dir_path( __FILE__ ) . "views/settings_installed.php");
  } else {
    include_once(plugin_dir_path( __FILE__ ) . "views/settings_install.php");
  }
}

function opochat_sync_wordpress_user() {
  $output = "";


  if (is_user_logged_in()) {
    $current_user = wp_get_current_user();
  }

  if (!isset($current_user)) {
    return "";
  }

  $website_verify = get_option("website_verify");

  $email = esc_js($current_user->user_email);
  $nickname = esc_js($current_user->display_name);

  if (!empty($email) && empty($website_verify)) {
    $output .= "\$opochat.push(['set', 'user:email', '" . $email . "']);";
  } else if (!empty($email)) {
    $hmac = hash_hmac("sha256", $email, $website_verify);
    $output .= "\$opochat.push(['set', 'user:email', ['" . $email . "', '" . $hmac . "']]);";
  }

  if (!empty($nickname)) {
    $output .= "\$opochat.push(['set', 'user:nickname', '" . $nickname . "']);";
  }

  return $output;
}

function opochat_sync_woocommerce_customer() {
  $output = "";
  if (!class_exists("WooCommerce") || is_admin()) {
    return $output;
  }

  if (WC()->session == NULL) {
    return $output;
  }

  $customer = WC()->session->get("customer");

  if ($customer == NULL) {
    return $output;
  }

  if (isset($customer["phone"]) && !empty($customer["phone"])) {
    $output .= "\$opochat.push(['set', 'user:phone', '" . $customer["phone"] . "']);";
  }

  $nickname = "";

  if (isset($customer["first_name"]) && !empty($customer["first_name"])) {
    $nickname = esc_js($customer["first_name"]);
  }
  if (isset($customer["last_name"]) && !empty($customer["last_name"])) {
    $nickname .= " ".(esc_js($customer["last_name"]));
  }

  if (!empty($nickname)) {
    $output .= "\$opochat.push(['set', 'user:nickname', '" . $nickname . "']);";
  }

  $data = array();
  $data_keys = array(
    "company",
    "address",
    "address_1",
    "address_2",
    "postcode",
    "state",
    "country",
    "shipping_company",
    "shipping_address",
    "shipping_address_1",
    "shipping_address_2",
    "shipping_state",
    "shipping_country",
  );

  foreach ($data_keys as $key) {
    if (isset($customer[$key]) && !empty($customer[$key])) {
      $data[] = "['". esc_js($key) . "', '" . esc_js($customer[$key]) . "']";
    }
  }

  if (count($data) > 0) {
    $output .= "\$opochat.push(['set', 'session:data', [[" . implode(",", $data) . "]]]);";
  }

  return $output;
}


function opochat_enqueue_scripts() {
  $website_apiKey = get_option("website_apiKey");
  $locale = str_replace("_", "-", strtolower(get_locale()));
  $locale = preg_replace("/([a-z]{2}-[a-z]{2})(-.*)/", "$1", $locale);

  if (empty($website_apiKey)) return;

  $output = "
    window.\$opochat = [];
    if (!window.OPOCHAT_RUNTIME_CONFIG) {
      window.OPOCHAT_RUNTIME_CONFIG = {};
    }
    if (!window.OPOCHAT_RUNTIME_CONFIG.locale) {
      window.OPOCHAT_RUNTIME_CONFIG.locale = '$locale';
    }
    OPOCHAT_API_KEY = '$website_apiKey';
  ";

  $output .= opochat_sync_wordpress_user();
  $output .= opochat_sync_woocommerce_customer();

  $cache_buster = gmdate("Ymd");

  wp_register_script("opochat", "https://opo.chat/code.js?apiKey=" . urlencode($website_apiKey), array(), $cache_buster, true);
  wp_enqueue_script("opochat");
  wp_add_inline_script("opochat", $output, "before");
}

function opochat_modify_script_tag($tag, $handle, $src) {
  if ($handle === 'opochat') {
    return '<script defer crossorigin="anonymous" src="' . esc_url($src) . '"></script>';
  }
  return $tag;
}
add_filter('script_loader_tag', 'opochat_modify_script_tag', 10, 3);

 